"""
This package implements the MVC paradigm using servey actions and jinja, allowing production of web pages.
It is an optional extra. The hosted version uses starlette, and the AWS version uses API gateway.
"""
